@@ -22,6 +22,8 @@ try:
 except ImportError:  # pragma: NO COVER
     import mock
 
+from collections.abc import Iterable
+import json
 import math
 
 from google.api_core import gapic_v1, grpc_helpers, grpc_helpers_async, path_template
@@ -31,12 +33,15 @@ import google.auth
 from google.auth import credentials as ga_credentials
 from google.auth.exceptions import MutualTLSChannelError
 from google.oauth2 import service_account
+from google.protobuf import json_format
 from google.type import postal_address_pb2  # type: ignore
 import grpc
 from grpc.experimental import aio
 from proto.marshal.rules import wrappers
 from proto.marshal.rules.dates import DurationRule, TimestampRule
 import pytest
+from requests import PreparedRequest, Request, Response
+from requests.sessions import Session
 
 from google.maps.addressvalidation_v1.services.address_validation import (
     AddressValidationAsyncClient,
@@ -96,6 +101,7 @@ def test__get_default_mtls_endpoint():
     [
         (AddressValidationClient, "grpc"),
         (AddressValidationAsyncClient, "grpc_asyncio"),
+        (AddressValidationClient, "rest"),
     ],
 )
 def test_address_validation_client_from_service_account_info(
@@ -111,7 +117,11 @@ def test_address_validation_client_from_service_account_info(
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == ("addressvalidation.googleapis.com:443")
+        assert client.transport._host == (
+            "addressvalidation.googleapis.com:443"
+            if transport_name in ["grpc", "grpc_asyncio"]
+            else "https://addressvalidation.googleapis.com"
+        )
 
 
 @pytest.mark.parametrize(
@@ -119,6 +129,7 @@ def test_address_validation_client_from_service_account_info(
     [
         (transports.AddressValidationGrpcTransport, "grpc"),
         (transports.AddressValidationGrpcAsyncIOTransport, "grpc_asyncio"),
+        (transports.AddressValidationRestTransport, "rest"),
     ],
 )
 def test_address_validation_client_service_account_always_use_jwt(
@@ -144,6 +155,7 @@ def test_address_validation_client_service_account_always_use_jwt(
     [
         (AddressValidationClient, "grpc"),
         (AddressValidationAsyncClient, "grpc_asyncio"),
+        (AddressValidationClient, "rest"),
     ],
 )
 def test_address_validation_client_from_service_account_file(
@@ -166,13 +178,18 @@ def test_address_validation_client_from_service_account_file(
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == ("addressvalidation.googleapis.com:443")
+        assert client.transport._host == (
+            "addressvalidation.googleapis.com:443"
+            if transport_name in ["grpc", "grpc_asyncio"]
+            else "https://addressvalidation.googleapis.com"
+        )
 
 
 def test_address_validation_client_get_transport_class():
     transport = AddressValidationClient.get_transport_class()
     available_transports = [
         transports.AddressValidationGrpcTransport,
+        transports.AddressValidationRestTransport,
     ]
     assert transport in available_transports
 
@@ -189,6 +206,7 @@ def test_address_validation_client_get_transport_class():
             transports.AddressValidationGrpcAsyncIOTransport,
             "grpc_asyncio",
         ),
+        (AddressValidationClient, transports.AddressValidationRestTransport, "rest"),
     ],
 )
 @mock.patch.object(
@@ -344,6 +362,18 @@ def test_address_validation_client_client_options(
             "grpc_asyncio",
             "false",
         ),
+        (
+            AddressValidationClient,
+            transports.AddressValidationRestTransport,
+            "rest",
+            "true",
+        ),
+        (
+            AddressValidationClient,
+            transports.AddressValidationRestTransport,
+            "rest",
+            "false",
+        ),
     ],
 )
 @mock.patch.object(
@@ -543,6 +573,7 @@ def test_address_validation_client_get_mtls_endpoint_and_cert_source(client_clas
             transports.AddressValidationGrpcAsyncIOTransport,
             "grpc_asyncio",
         ),
+        (AddressValidationClient, transports.AddressValidationRestTransport, "rest"),
     ],
 )
 def test_address_validation_client_client_options_scopes(
@@ -583,6 +614,12 @@ def test_address_validation_client_client_options_scopes(
             "grpc_asyncio",
             grpc_helpers_async,
         ),
+        (
+            AddressValidationClient,
+            transports.AddressValidationRestTransport,
+            "rest",
+            None,
+        ),
     ],
 )
 def test_address_validation_client_client_options_credentials_file(
@@ -881,6 +918,454 @@ async def test_provide_validation_feedback_async_from_dict():
     await test_provide_validation_feedback_async(request_type=dict)
 
 
+@pytest.mark.parametrize(
+    "request_type",
+    [
+        address_validation_service.ValidateAddressRequest,
+        dict,
+    ],
+)
+def test_validate_address_rest(request_type):
+    client = AddressValidationClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = address_validation_service.ValidateAddressResponse(
+            response_id="response_id_value",
+        )
+
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 200
+        pb_return_value = address_validation_service.ValidateAddressResponse.pb(
+            return_value
+        )
+        json_return_value = json_format.MessageToJson(pb_return_value)
+
+        response_value._content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+        response = client.validate_address(request)
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(response, address_validation_service.ValidateAddressResponse)
+    assert response.response_id == "response_id_value"
+
+
+def test_validate_address_rest_required_fields(
+    request_type=address_validation_service.ValidateAddressRequest,
+):
+    transport_class = transports.AddressValidationRestTransport
+
+    request_init = {}
+    request = request_type(**request_init)
+    pb_request = request_type.pb(request)
+    jsonified_request = json.loads(
+        json_format.MessageToJson(
+            pb_request,
+            including_default_value_fields=False,
+            use_integers_for_enums=False,
+        )
+    )
+
+    # verify fields with default values are dropped
+
+    unset_fields = transport_class(
+        credentials=ga_credentials.AnonymousCredentials()
+    ).validate_address._get_unset_required_fields(jsonified_request)
+    jsonified_request.update(unset_fields)
+
+    # verify required fields with default values are now present
+
+    unset_fields = transport_class(
+        credentials=ga_credentials.AnonymousCredentials()
+    ).validate_address._get_unset_required_fields(jsonified_request)
+    jsonified_request.update(unset_fields)
+
+    # verify required fields with non-default values are left alone
+
+    client = AddressValidationClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+    request = request_type(**request_init)
+
+    # Designate an appropriate value for the returned response.
+    return_value = address_validation_service.ValidateAddressResponse()
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(Session, "request") as req:
+        # We need to mock transcode() because providing default values
+        # for required fields will fail the real version if the http_options
+        # expect actual values for those fields.
+        with mock.patch.object(path_template, "transcode") as transcode:
+            # A uri without fields and an empty body will force all the
+            # request fields to show up in the query_params.
+            pb_request = request_type.pb(request)
+            transcode_result = {
+                "uri": "v1/sample_method",
+                "method": "post",
+                "query_params": pb_request,
+            }
+            transcode_result["body"] = pb_request
+            transcode.return_value = transcode_result
+
+            response_value = Response()
+            response_value.status_code = 200
+
+            pb_return_value = address_validation_service.ValidateAddressResponse.pb(
+                return_value
+            )
+            json_return_value = json_format.MessageToJson(pb_return_value)
+
+            response_value._content = json_return_value.encode("UTF-8")
+            req.return_value = response_value
+
+            response = client.validate_address(request)
+
+            expected_params = [("$alt", "json;enum-encoding=int")]
+            actual_params = req.call_args.kwargs["params"]
+            assert expected_params == actual_params
+
+
+def test_validate_address_rest_unset_required_fields():
+    transport = transports.AddressValidationRestTransport(
+        credentials=ga_credentials.AnonymousCredentials
+    )
+
+    unset_fields = transport.validate_address._get_unset_required_fields({})
+    assert set(unset_fields) == (set(()) & set(("address",)))
+
+
+@pytest.mark.parametrize("null_interceptor", [True, False])
+def test_validate_address_rest_interceptors(null_interceptor):
+    transport = transports.AddressValidationRestTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+        interceptor=None
+        if null_interceptor
+        else transports.AddressValidationRestInterceptor(),
+    )
+    client = AddressValidationClient(transport=transport)
+    with mock.patch.object(
+        type(client.transport._session), "request"
+    ) as req, mock.patch.object(
+        path_template, "transcode"
+    ) as transcode, mock.patch.object(
+        transports.AddressValidationRestInterceptor, "post_validate_address"
+    ) as post, mock.patch.object(
+        transports.AddressValidationRestInterceptor, "pre_validate_address"
+    ) as pre:
+        pre.assert_not_called()
+        post.assert_not_called()
+        pb_message = address_validation_service.ValidateAddressRequest.pb(
+            address_validation_service.ValidateAddressRequest()
+        )
+        transcode.return_value = {
+            "method": "post",
+            "uri": "my_uri",
+            "body": pb_message,
+            "query_params": pb_message,
+        }
+
+        req.return_value = Response()
+        req.return_value.status_code = 200
+        req.return_value.request = PreparedRequest()
+        req.return_value._content = (
+            address_validation_service.ValidateAddressResponse.to_json(
+                address_validation_service.ValidateAddressResponse()
+            )
+        )
+
+        request = address_validation_service.ValidateAddressRequest()
+        metadata = [
+            ("key", "val"),
+            ("cephalopod", "squid"),
+        ]
+        pre.return_value = request, metadata
+        post.return_value = address_validation_service.ValidateAddressResponse()
+
+        client.validate_address(
+            request,
+            metadata=[
+                ("key", "val"),
+                ("cephalopod", "squid"),
+            ],
+        )
+
+        pre.assert_called_once()
+        post.assert_called_once()
+
+
+def test_validate_address_rest_bad_request(
+    transport: str = "rest",
+    request_type=address_validation_service.ValidateAddressRequest,
+):
+    client = AddressValidationClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a BadRequest error.
+    with mock.patch.object(Session, "request") as req, pytest.raises(
+        core_exceptions.BadRequest
+    ):
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 400
+        response_value.request = Request()
+        req.return_value = response_value
+        client.validate_address(request)
+
+
+def test_validate_address_rest_error():
+    client = AddressValidationClient(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
+    )
+
+
+@pytest.mark.parametrize(
+    "request_type",
+    [
+        address_validation_service.ProvideValidationFeedbackRequest,
+        dict,
+    ],
+)
+def test_provide_validation_feedback_rest(request_type):
+    client = AddressValidationClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(type(client.transport._session), "request") as req:
+        # Designate an appropriate value for the returned response.
+        return_value = address_validation_service.ProvideValidationFeedbackResponse()
+
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 200
+        pb_return_value = (
+            address_validation_service.ProvideValidationFeedbackResponse.pb(
+                return_value
+            )
+        )
+        json_return_value = json_format.MessageToJson(pb_return_value)
+
+        response_value._content = json_return_value.encode("UTF-8")
+        req.return_value = response_value
+        response = client.provide_validation_feedback(request)
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(
+        response, address_validation_service.ProvideValidationFeedbackResponse
+    )
+
+
+def test_provide_validation_feedback_rest_required_fields(
+    request_type=address_validation_service.ProvideValidationFeedbackRequest,
+):
+    transport_class = transports.AddressValidationRestTransport
+
+    request_init = {}
+    request_init["response_id"] = ""
+    request = request_type(**request_init)
+    pb_request = request_type.pb(request)
+    jsonified_request = json.loads(
+        json_format.MessageToJson(
+            pb_request,
+            including_default_value_fields=False,
+            use_integers_for_enums=False,
+        )
+    )
+
+    # verify fields with default values are dropped
+
+    unset_fields = transport_class(
+        credentials=ga_credentials.AnonymousCredentials()
+    ).provide_validation_feedback._get_unset_required_fields(jsonified_request)
+    jsonified_request.update(unset_fields)
+
+    # verify required fields with default values are now present
+
+    jsonified_request["responseId"] = "response_id_value"
+
+    unset_fields = transport_class(
+        credentials=ga_credentials.AnonymousCredentials()
+    ).provide_validation_feedback._get_unset_required_fields(jsonified_request)
+    jsonified_request.update(unset_fields)
+
+    # verify required fields with non-default values are left alone
+    assert "responseId" in jsonified_request
+    assert jsonified_request["responseId"] == "response_id_value"
+
+    client = AddressValidationClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport="rest",
+    )
+    request = request_type(**request_init)
+
+    # Designate an appropriate value for the returned response.
+    return_value = address_validation_service.ProvideValidationFeedbackResponse()
+    # Mock the http request call within the method and fake a response.
+    with mock.patch.object(Session, "request") as req:
+        # We need to mock transcode() because providing default values
+        # for required fields will fail the real version if the http_options
+        # expect actual values for those fields.
+        with mock.patch.object(path_template, "transcode") as transcode:
+            # A uri without fields and an empty body will force all the
+            # request fields to show up in the query_params.
+            pb_request = request_type.pb(request)
+            transcode_result = {
+                "uri": "v1/sample_method",
+                "method": "post",
+                "query_params": pb_request,
+            }
+            transcode_result["body"] = pb_request
+            transcode.return_value = transcode_result
+
+            response_value = Response()
+            response_value.status_code = 200
+
+            pb_return_value = (
+                address_validation_service.ProvideValidationFeedbackResponse.pb(
+                    return_value
+                )
+            )
+            json_return_value = json_format.MessageToJson(pb_return_value)
+
+            response_value._content = json_return_value.encode("UTF-8")
+            req.return_value = response_value
+
+            response = client.provide_validation_feedback(request)
+
+            expected_params = [("$alt", "json;enum-encoding=int")]
+            actual_params = req.call_args.kwargs["params"]
+            assert expected_params == actual_params
+
+
+def test_provide_validation_feedback_rest_unset_required_fields():
+    transport = transports.AddressValidationRestTransport(
+        credentials=ga_credentials.AnonymousCredentials
+    )
+
+    unset_fields = transport.provide_validation_feedback._get_unset_required_fields({})
+    assert set(unset_fields) == (
+        set(())
+        & set(
+            (
+                "conclusion",
+                "responseId",
+            )
+        )
+    )
+
+
+@pytest.mark.parametrize("null_interceptor", [True, False])
+def test_provide_validation_feedback_rest_interceptors(null_interceptor):
+    transport = transports.AddressValidationRestTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+        interceptor=None
+        if null_interceptor
+        else transports.AddressValidationRestInterceptor(),
+    )
+    client = AddressValidationClient(transport=transport)
+    with mock.patch.object(
+        type(client.transport._session), "request"
+    ) as req, mock.patch.object(
+        path_template, "transcode"
+    ) as transcode, mock.patch.object(
+        transports.AddressValidationRestInterceptor, "post_provide_validation_feedback"
+    ) as post, mock.patch.object(
+        transports.AddressValidationRestInterceptor, "pre_provide_validation_feedback"
+    ) as pre:
+        pre.assert_not_called()
+        post.assert_not_called()
+        pb_message = address_validation_service.ProvideValidationFeedbackRequest.pb(
+            address_validation_service.ProvideValidationFeedbackRequest()
+        )
+        transcode.return_value = {
+            "method": "post",
+            "uri": "my_uri",
+            "body": pb_message,
+            "query_params": pb_message,
+        }
+
+        req.return_value = Response()
+        req.return_value.status_code = 200
+        req.return_value.request = PreparedRequest()
+        req.return_value._content = (
+            address_validation_service.ProvideValidationFeedbackResponse.to_json(
+                address_validation_service.ProvideValidationFeedbackResponse()
+            )
+        )
+
+        request = address_validation_service.ProvideValidationFeedbackRequest()
+        metadata = [
+            ("key", "val"),
+            ("cephalopod", "squid"),
+        ]
+        pre.return_value = request, metadata
+        post.return_value = (
+            address_validation_service.ProvideValidationFeedbackResponse()
+        )
+
+        client.provide_validation_feedback(
+            request,
+            metadata=[
+                ("key", "val"),
+                ("cephalopod", "squid"),
+            ],
+        )
+
+        pre.assert_called_once()
+        post.assert_called_once()
+
+
+def test_provide_validation_feedback_rest_bad_request(
+    transport: str = "rest",
+    request_type=address_validation_service.ProvideValidationFeedbackRequest,
+):
+    client = AddressValidationClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+        transport=transport,
+    )
+
+    # send a request that will satisfy transcoding
+    request_init = {}
+    request = request_type(**request_init)
+
+    # Mock the http request call within the method and fake a BadRequest error.
+    with mock.patch.object(Session, "request") as req, pytest.raises(
+        core_exceptions.BadRequest
+    ):
+        # Wrap the value into a proper Response obj
+        response_value = Response()
+        response_value.status_code = 400
+        response_value.request = Request()
+        req.return_value = response_value
+        client.provide_validation_feedback(request)
+
+
+def test_provide_validation_feedback_rest_error():
+    client = AddressValidationClient(
+        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
+    )
+
+
 def test_credentials_transport_error():
     # It is an error to provide credentials and a transport instance.
     transport = transports.AddressValidationGrpcTransport(
@@ -962,6 +1447,7 @@ def test_transport_get_channel():
     [
         transports.AddressValidationGrpcTransport,
         transports.AddressValidationGrpcAsyncIOTransport,
+        transports.AddressValidationRestTransport,
     ],
 )
 def test_transport_adc(transport_class):
@@ -976,6 +1462,7 @@ def test_transport_adc(transport_class):
     "transport_name",
     [
         "grpc",
+        "rest",
     ],
 )
 def test_transport_kind(transport_name):
@@ -1106,6 +1593,7 @@ def test_address_validation_transport_auth_adc(transport_class):
     [
         transports.AddressValidationGrpcTransport,
         transports.AddressValidationGrpcAsyncIOTransport,
+        transports.AddressValidationRestTransport,
     ],
 )
 def test_address_validation_transport_auth_gdch_credentials(transport_class):
@@ -1203,11 +1691,23 @@ def test_address_validation_grpc_transport_client_cert_source_for_mtls(transport
             )
 
 
+def test_address_validation_http_transport_client_cert_source_for_mtls():
+    cred = ga_credentials.AnonymousCredentials()
+    with mock.patch(
+        "google.auth.transport.requests.AuthorizedSession.configure_mtls_channel"
+    ) as mock_configure_mtls_channel:
+        transports.AddressValidationRestTransport(
+            credentials=cred, client_cert_source_for_mtls=client_cert_source_callback
+        )
+        mock_configure_mtls_channel.assert_called_once_with(client_cert_source_callback)
+
+
 @pytest.mark.parametrize(
     "transport_name",
     [
         "grpc",
         "grpc_asyncio",
+        "rest",
     ],
 )
 def test_address_validation_host_no_port(transport_name):
@@ -1218,7 +1718,11 @@ def test_address_validation_host_no_port(transport_name):
         ),
         transport=transport_name,
     )
-    assert client.transport._host == ("addressvalidation.googleapis.com:443")
+    assert client.transport._host == (
+        "addressvalidation.googleapis.com:443"
+        if transport_name in ["grpc", "grpc_asyncio"]
+        else "https://addressvalidation.googleapis.com"
+    )
 
 
 @pytest.mark.parametrize(
@@ -1226,6 +1730,7 @@ def test_address_validation_host_no_port(transport_name):
     [
         "grpc",
         "grpc_asyncio",
+        "rest",
     ],
 )
 def test_address_validation_host_with_port(transport_name):
@@ -1236,7 +1741,36 @@ def test_address_validation_host_with_port(transport_name):
         ),
         transport=transport_name,
     )
-    assert client.transport._host == ("addressvalidation.googleapis.com:8000")
+    assert client.transport._host == (
+        "addressvalidation.googleapis.com:8000"
+        if transport_name in ["grpc", "grpc_asyncio"]
+        else "https://addressvalidation.googleapis.com:8000"
+    )
+
+
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "rest",
+    ],
+)
+def test_address_validation_client_transport_session_collision(transport_name):
+    creds1 = ga_credentials.AnonymousCredentials()
+    creds2 = ga_credentials.AnonymousCredentials()
+    client1 = AddressValidationClient(
+        credentials=creds1,
+        transport=transport_name,
+    )
+    client2 = AddressValidationClient(
+        credentials=creds2,
+        transport=transport_name,
+    )
+    session1 = client1.transport.validate_address._session
+    session2 = client2.transport.validate_address._session
+    assert session1 != session2
+    session1 = client1.transport.provide_validation_feedback._session
+    session2 = client2.transport.provide_validation_feedback._session
+    assert session1 != session2
 
 
 def test_address_validation_grpc_transport_channel():
@@ -1507,6 +2041,7 @@ async def test_transport_close_async():
 
 def test_transport_close():
     transports = {
+        "rest": "_session",
         "grpc": "_grpc_channel",
     }
 
@@ -1524,6 +2059,7 @@ def test_transport_close():
 
 def test_client_ctx():
     transports = [
+        "rest",
         "grpc",
     ]
     for transport in transports:
